@@ -198,7 +198,7 @@ class Auth:
             self._rest._connection._internal_application_name,
             self._rest._connection._internal_application_version,
             self._rest._connection._ocsp_mode(),
-            self._rest._connection._login_timeout,
+            self._rest._connection.login_timeout,
             self._rest._connection._network_timeout,
             self._rest._connection._socket_timeout,
         )
@@ -417,27 +417,35 @@ class Auth:
         else:
             logger.debug(
                 "token = %s",
-                "******"
-                if ret["data"] and ret["data"].get("token") is not None
-                else "NULL",
+                (
+                    "******"
+                    if ret["data"] and ret["data"].get("token") is not None
+                    else "NULL"
+                ),
             )
             logger.debug(
                 "master_token = %s",
-                "******"
-                if ret["data"] and ret["data"].get("masterToken") is not None
-                else "NULL",
+                (
+                    "******"
+                    if ret["data"] and ret["data"].get("masterToken") is not None
+                    else "NULL"
+                ),
             )
             logger.debug(
                 "id_token = %s",
-                "******"
-                if ret["data"] and ret["data"].get("idToken") is not None
-                else "NULL",
+                (
+                    "******"
+                    if ret["data"] and ret["data"].get("idToken") is not None
+                    else "NULL"
+                ),
             )
             logger.debug(
                 "mfa_token = %s",
-                "******"
-                if ret["data"] and ret["data"].get("mfaToken") is not None
-                else "NULL",
+                (
+                    "******"
+                    if ret["data"] and ret["data"].get("mfaToken") is not None
+                    else "NULL"
+                ),
             )
             if not ret["data"]:
                 Error.errorhandler_wrapper(
