@@ -149,9 +149,11 @@ class ResultMetadata(NamedTuple):
     def from_column(cls, col: dict[str, Any]):
         """Initializes a ResultMetadata object from the column description in the query response."""
         type_code = FIELD_NAME_TO_ID[
-            col["extTypeName"].upper()
-            if col.get("extTypeName")
-            else col["type"].upper()
+            (
+                col["extTypeName"].upper()
+                if col.get("extTypeName")
+                else col["type"].upper()
+            )
         ]
 
         return cls(
@@ -546,12 +548,10 @@ class SnowflakeCursor:
         return None
 
     @overload
-    def callproc(self, procname: str) -> tuple:
-        ...
+    def callproc(self, procname: str) -> tuple: ...
 
     @overload
-    def callproc(self, procname: str, args: T) -> T:
-        ...
+    def callproc(self, procname: str, args: T) -> T: ...
 
     def callproc(self, procname: str, args=tuple()):
         """Call a stored procedure.
@@ -601,6 +601,7 @@ class SnowflakeCursor:
         _no_results: bool = False,
         _is_put_get=None,
         _no_retry: bool = False,
+        dataframe_ast: str | None = None,
     ) -> dict[str, Any]:
         del self.messages[:]
 
@@ -630,9 +631,9 @@ class SnowflakeCursor:
                 if str(result_format_val).upper() == "ARROW":
                     self.check_can_use_arrow_resultset()
                 elif result_format_val is None:
-                    statement_params[
-                        PARAMETER_PYTHON_CONNECTOR_QUERY_RESULT_FORMAT
-                    ] = "JSON"
+                    statement_params[PARAMETER_PYTHON_CONNECTOR_QUERY_RESULT_FORMAT] = (
+                        "JSON"
+                    )
 
         self._sequence_counter = self._connection._next_sequence_counter()
         self._request_id = uuid.uuid4()
@@ -704,6 +705,7 @@ class SnowflakeCursor:
                 _no_results=_no_results,
                 _no_retry=_no_retry,
                 timeout=real_timeout,
+                dataframe_ast=dataframe_ast,
             )
         finally:
             try:
@@ -758,9 +760,11 @@ class SnowflakeCursor:
         if params is not None and len(params) == 0:
             self._log_telemetry_job_data(
                 TelemetryField.EMPTY_SEQ_INTERPOLATION,
-                TelemetryData.TRUE
-                if self.connection._interpolate_empty_sequences
-                else TelemetryData.FALSE,
+                (
+                    TelemetryData.TRUE
+                    if self.connection._interpolate_empty_sequences
+                    else TelemetryData.FALSE
+                ),
             )
         if logger.getEffectiveLevel() <= logging.DEBUG:
             logger.debug(
@@ -807,8 +811,8 @@ class SnowflakeCursor:
         _skip_upload_on_content_match: bool = False,
         file_stream: IO[bytes] | None = None,
         num_statements: int | None = None,
-    ) -> Self | None:
-        ...
+        _dataframe_ast: str | None = None,
+    ) -> Self | None: ...
 
     @overload
     def execute(
@@ -837,8 +841,8 @@ class SnowflakeCursor:
         _skip_upload_on_content_match: bool = False,
         file_stream: IO[bytes] | None = None,
         num_statements: int | None = None,
-    ) -> dict[str, Any] | None:
-        ...
+        _dataframe_ast: str | None = None,
+    ) -> dict[str, Any] | None: ...
 
     def execute(
         self,
@@ -866,6 +870,7 @@ class SnowflakeCursor:
         _skip_upload_on_content_match: bool = False,
         file_stream: IO[bytes] | None = None,
         num_statements: int | None = None,
+        _dataframe_ast: str | None = None,
     ) -> Self | dict[str, Any] | None:
         """Executes a command/query.
 
@@ -900,6 +905,7 @@ class SnowflakeCursor:
             file_stream: File-like object to be uploaded with PUT
             num_statements: Query level parameter submitted in _statement_params constraining exact number of
             statements being submitted (or 0 if submitting an uncounted number) when using a multi-statement query.
+            _dataframe_ast: Base64-encoded dataframe request abstract syntax tree.
 
         Returns:
             The cursor itself, or None if some error happened, or the response returned
@@ -941,6 +947,7 @@ class SnowflakeCursor:
             "_no_results": _no_results,
             "_is_put_get": _is_put_get,
             "_no_retry": _no_retry,
+            "dataframe_ast": _dataframe_ast,
         }
 
         if self._connection.is_pyformat:
@@ -1278,12 +1285,10 @@ class SnowflakeCursor:
         return self._result_set._fetch_arrow_batches()
 
     @overload
-    def fetch_arrow_all(self, force_return_table: Literal[False]) -> Table | None:
-        ...
+    def fetch_arrow_all(self, force_return_table: Literal[False]) -> Table | None: ...
 
     @overload
-    def fetch_arrow_all(self, force_return_table: Literal[True]) -> Table:
-        ...
+    def fetch_arrow_all(self, force_return_table: Literal[True]) -> Table: ...
 
     def fetch_arrow_all(self, force_return_table: bool = False) -> Table | None:
         """
@@ -1314,7 +1319,17 @@ class SnowflakeCursor:
         return self._result_set._fetch_pandas_batches(**kwargs)
 
     def fetch_pandas_all(self, **kwargs: Any) -> DataFrame:
-        """Fetch Pandas dataframes in batches, where 'batch' refers to Snowflake Chunk."""
+        """
+        Fetch Pandas dataframes in batches, where 'batch' refers to Snowflake Chunk.
+
+        Returns:
+            A pandas dataframe containing the cursors query results.
+
+        Note:
+            Timestamp types that are nested in structured types may not be accurately represented
+            due to a limitation in the pyarrow to pandas conversion.
+            See here for more information: https://github.com/apache/arrow/issues/4116
+        """
         self.check_can_use_pandas()
         if self._prefetch_hook is not None:
             self._prefetch_hook()
@@ -1623,7 +1638,8 @@ class SnowflakeCursor:
             no_data_counter = 0
             retry_pattern_pos = 0
             while True:
-                status = self.connection.get_query_status(sfqid)
+                status, status_resp = self.connection._get_query_status(sfqid)
+                self.connection._cache_query_status(sfqid, status)
                 if not self.connection.is_still_running(status):
                     break
                 if status == QueryStatus.NO_DATA:  # pragma: no cover
@@ -1640,10 +1656,12 @@ class SnowflakeCursor:
                 if retry_pattern_pos < (len(ASYNC_RETRY_PATTERN) - 1):
                     retry_pattern_pos += 1
             if status != QueryStatus.SUCCESS:
-                raise DatabaseError(
-                    "Status of query '{}' is {}, results are unavailable".format(
-                        sfqid, status.name
-                    )
+                logger.info(f"Status of query '{sfqid}' is {status.name}")
+                self.connection._process_error_query_status(
+                    sfqid,
+                    status_resp,
+                    error_message=f"Status of query '{sfqid}' is {status.name}, results are unavailable",
+                    error_cls=DatabaseError,
                 )
             self._inner_cursor.execute(f"select * from table(result_scan('{sfqid}'))")
             self._result = self._inner_cursor._result
